package com.github.gkutiel.filter;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import javax.xml.bind.annotation.adapters.HexBinaryAdapter;

import org.apache.commons.io.IOUtils;

public enum Auth {
	;
	private static final KeyPair KEY_PAIR;
	private static final HexBinaryAdapter ADAPTER = new HexBinaryAdapter();
	private static final String SIG_ALG = "SHA1withDSA";

	static {
		try {
			final KeyPair tmp = loadKeys();
			if (tmp != null) KEY_PAIR = tmp;
			else {
				KEY_PAIR = KeyPairGenerator.getInstance("DSA").generateKeyPair();
				saveKeys(KEY_PAIR);
			}
		} catch (final NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		}
	}

	public static String md5(final byte[] data) {
		try {
			return ADAPTER.marshal(MessageDigest.getInstance("MD5").digest(data));
		} catch (final NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		}
	}

	public static String md5(final String data) {
		return md5(data.getBytes());
	}

	public static String sign(final byte[] data) {
		try {
			final Signature sig = Signature.getInstance(SIG_ALG);
			sig.initSign(KEY_PAIR.getPrivate());
			sig.update(data);
			return ADAPTER.marshal(sig.sign());
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static String sign(final String data) {
		return sign(data.getBytes());
	}

	public static boolean verify(final byte[] data, final String sign) {
		try {
			final Signature sig = Signature.getInstance(SIG_ALG);
			sig.initVerify(KEY_PAIR.getPublic());
			sig.update(data);
			return sig.verify(ADAPTER.unmarshal(sign));
		} catch (final Exception e) {
			return false;
		}
	}

	public static boolean verify(final String data, final String sign) {
		return verify(data.getBytes(), sign);
	}

	private static KeyPair loadKeys() {
		try (final FileInputStream inPub = new FileInputStream("key.pub"); FileInputStream inPri = new FileInputStream("key.pri")) {
			final KeyFactory keyFactory = KeyFactory.getInstance("DSA");
			final PublicKey pub = keyFactory.generatePublic(new X509EncodedKeySpec(IOUtils.toByteArray(inPub)));
			final PrivateKey pri = keyFactory.generatePrivate(new PKCS8EncodedKeySpec(IOUtils.toByteArray(inPri)));
			return new KeyPair(pub, pri);
		} catch (final Exception e) {
			return null;
		}
	}

	private static void saveKeys(final KeyPair keyPair) {
		try {
			new FileOutputStream("key.pub").write(new X509EncodedKeySpec(keyPair.getPublic().getEncoded()).getEncoded());
			new FileOutputStream("key.pri").write(new PKCS8EncodedKeySpec(keyPair.getPrivate().getEncoded()).getEncoded());
		} catch (final IOException e) {
			throw new RuntimeException(e);
		}
	}
}
