package com.github.gkutiel.filter;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.github.gkutiel.filter.Injector.MultipartInjector;
import com.github.gkutiel.filter.Injector.UrlEncodedInjector;

class Engine {

	private final HttpServletRequest req;
	private final HttpServletResponse res;

	public Engine(final HttpServletRequest req, final HttpServletResponse res) {
		this.req = req;
		this.res = res;
	}

	public void invoke(final Class<? extends Filter> filterClass) {
		invoke(getInjector(), null, filterChain(filterClass));
	}

	private List<Class<? extends Filter>> filterChain(final Class<?> c) {
		if (c == null) return new ArrayList<>();
		final List<Class<? extends Filter>> classChain = filterChain(c.getEnclosingClass());
		if (Filter.class.isAssignableFrom(c)) classChain.add(c.asSubclass(Filter.class));
		return classChain;
	}

	private Injector getInjector() {
		final String contentType = req.getContentType();
		if (contentType != null && contentType.startsWith("multipart/form-data;")) return new MultipartInjector(req, res);
		return new UrlEncodedInjector(req, res);
	}

	private void invoke(final Injector injector, final Filter outer, final List<Class<? extends Filter>> chain) {
		if (outer != null) outer.apply();
		if (chain.isEmpty()) return;
		final Class<? extends Filter> filterClass = chain.remove(0);
		invoke(injector, newFilter(injector, outer, filterClass), chain);
	}

	private Filter newFilter(final Filter outer, final Class<? extends Filter> filterClass) {
		try {
			if (outer == null) {
				final Constructor<? extends Filter> ctor = filterClass.getDeclaredConstructor();
				ctor.setAccessible(true);
				return ctor.newInstance();
			}
			final Constructor<? extends Filter> ctor = filterClass.getDeclaredConstructor(outer.getClass());
			ctor.setAccessible(true);
			return ctor.newInstance(outer);
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

	private Filter newFilter(final Injector injector, final Filter outer, final Class<? extends Filter> filterClass) {
		try {
			return injector.inject(newFilter(outer, filterClass));
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}
}
