package com.github.gkutiel.filter;

import static com.github.gkutiel.filter.Util.getCookie;
import static java.lang.reflect.AccessibleObject.setAccessible;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;

import com.github.gkutiel.filter.Filter.Cookie;
import com.github.gkutiel.filter.Filter.Param;
import com.github.gkutiel.filter.Filter.SignedValue;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;

abstract class Injector {
	public static class MultipartInjector extends Injector {

		final private Map<String, FileItem> fileItems = new HashMap<>();

		MultipartInjector(final HttpServletRequest req, final HttpServletResponse res) {
			super(req, res);
			try {
				for (final FileItem fileItem : new ServletFileUpload(new DiskFileItemFactory()).parseRequest(req))
					fileItems.put(fileItem.getFieldName(), fileItem);
			} catch (final Exception e) {
				throw new RuntimeException(e);
			}
		}

		@Override
		FileItem getFileItem(final String name) {
			final FileItem fileItem = fileItems.get(name);
			if (fileItem == null || fileItem.isFormField()) return null;
			return fileItem;
		}

		@Override
		String getVal(final String name) {
			final FileItem fileItem = fileItems.get(name);
			if (fileItem == null || !fileItem.isFormField()) return null;
			return fileItem.getString();
		}
	}

	public static class UrlEncodedInjector extends Injector {

		UrlEncodedInjector(final HttpServletRequest req, final HttpServletResponse res) {
			super(req, res);
		}

		@Override
		FileItem getFileItem(@SuppressWarnings("unused") final String name) {
			return null;
		}

		@Override
		String getVal(final String name) {
			return req.getParameter(name);
		}
	}

	protected static final Gson GSON = new Gson();

	private static List<Field> fields(final Class<?> c) {
		if (c == null) return new ArrayList<>();
		final List<Field> fields = fields(c.getSuperclass());
		fields.addAll(Arrays.asList(c.getDeclaredFields()));
		return fields;
	}

	protected final HttpServletRequest req;
	private final HttpServletResponse res;

	Injector(final HttpServletRequest req, final HttpServletResponse res) {
		this.req = req;
		this.res = res;
	}

	private void injectCookie(final Filter filter, final Field field) throws JsonSyntaxException, IllegalArgumentException, IllegalAccessException {
		final String val = getCookie(req, field.getName());
		if (val == null) return;
		final SignedValue signedValue = GSON.fromJson(val, SignedValue.class);
		if (!signedValue.isValid()) return;
		field.set(filter, GSON.fromJson(signedValue.val(), field.getType()));
	}

	private void injectVal(final Filter filter, final Field field) throws JsonSyntaxException, IllegalArgumentException, IllegalAccessException {
		final String name = field.getName();
		final Class<?> type = field.getType();
		if (type.equals(FileItem.class)) {
			final FileItem fileItem = getFileItem(name);
			if (fileItem == null) return;
			field.set(filter, fileItem);
		}

		final String val = getVal(name);
		if (val == null) return;
		field.set(filter, GSON.fromJson(val, type));
	}

	abstract FileItem getFileItem(String name);

	abstract String getVal(String name);

	Filter inject(final Filter filter) throws IllegalArgumentException, IllegalAccessException {
		final List<Field> fields = fields(filter.getClass());
		setAccessible(fields.toArray(new Field[fields.size()]), true);

		for (final Field field : fields)
			if (field.getType().equals(HttpServletRequest.class)) field.set(filter, req);
			else if (field.getType().equals(HttpServletResponse.class)) field.set(filter, res);
			else if (field.isAnnotationPresent(Param.class)) injectVal(filter, field);
			else if (field.isAnnotationPresent(Cookie.class)) injectCookie(filter, field);

		return filter;
	}

}
